// File: Car.h
//
// Written by Richard Chang for CMSC 202 Spring 2007
//

#ifndef CAR_H
#define CAR_H

#include "proj2.h"
//#include "RaceCourse.h"

class RaceCourse ;


class Car {

public:

   // Constructors
   Car() ;
   Car(RaceCourse *ptr, int i, unsigned int fuel) ;


   // Moving
   void ZoomLeft() ;
   void ZoomRight() ;
   void ZoomUp() ;
   void ZoomDown() ;

   // "Accessors"
   Position GetPosition() const ;
   unsigned int GetFuelLevel() const ;

private:
   void SetFuelLevel(unsigned int f) ;

   unsigned int FuelLevel ;
   RaceCourse *Rptr ;
   int index ;
} ;



#endif
